/* Copyright (c) 2009 Peter Troshin
 *  
 *  JAva Bioinformatics Analysis Web Services (JABAWS) @version: 1.0
 * 
 *  This library is free software; you can redistribute it and/or modify it under the terms of the
 *  Apache License version 2 as published by the Apache Software Foundation
 * 
 *  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Apache 
 *  License for more details.
 * 
 *  A copy of the license is in apache_license.txt. It is also available here:
 * @see: http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 */

package compbio.metadata;

import java.util.List;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;

import compbio.data.sequence.FastaSequence;
import compbio.util.SysPrefs;

/**
 * A value object containing a maximum number of sequences and a maximum average
 * sequence length for a preset.
 * 
 * @see LimitsManager
 * 
 * @author pvtroshin
 * 
 *         Date January 2010
 * 
 * @param <T>
 *            the type of an executable for which this limit is defined.
 * 
 */
@XmlAccessorType(XmlAccessType.FIELD)
public class Limit<T> {

    private String preset;
    private int seqNumber;
    private int seqLength;

    @XmlAttribute
    boolean isDefault;

    private Limit() {
	// JAXB default constructor
    }

    public Limit(int seqNumber, int seqLength, String preset) {
	if (seqNumber <= 0) {
	    throw new IllegalArgumentException(
		    "seqNumber - a maximum number of sequences to align must be greater than 0. Value given:"
			    + seqNumber);
	}
	if (seqLength <= 0) {
	    throw new IllegalArgumentException(
		    "seqLength - an average sequence length must be greater than 0. Value given:"
			    + seqLength);
	}
	this.seqNumber = seqNumber;
	this.seqLength = seqLength;
	this.preset = preset;
	this.isDefault = false;
    }

    public Limit(int seqNumber, int seqLength, String preset, boolean isDefault) {
	this(seqNumber, seqNumber, preset);
	this.isDefault = isDefault;
    }

    public String getPreset() {
	return preset;
    }

    public int getAvgSeqLength() {
	return seqLength;
    }

    public int getSeqNumber() {
	return seqNumber;
    }

    public boolean isDefault() {
	return isDefault;
    }

    @Override
    public int hashCode() {
	final int prime = 31;
	int result = 1;
	result = prime * result + ((preset == null) ? 0 : preset.hashCode());
	result = prime * result + seqLength;
	result = prime * result + seqNumber;
	return result;
    }

    @Override
    public boolean equals(Object obj) {
	if (this == obj)
	    return true;
	if (obj == null)
	    return false;
	if (getClass() != obj.getClass())
	    return false;
	Limit other = (Limit) obj;
	if (preset == null) {
	    if (other.preset != null)
		return false;
	} else if (!preset.equals(other.preset))
	    return false;
	if (seqLength != other.seqLength)
	    return false;
	if (seqNumber != other.seqNumber)
	    return false;
	return true;
    }

    @Override
    public String toString() {
	String value = "";
	if (isDefault) {
	    value = "Default Limit" + SysPrefs.newlinechar;
	} else {
	    value = "Limit for Preset '" + preset + "'" + SysPrefs.newlinechar;
	}
	value += "Maximum sequence number=" + seqNumber + SysPrefs.newlinechar;
	value += "Average sequence length=" + seqLength + SysPrefs.newlinechar;
	value += SysPrefs.newlinechar;
	return value;
    }

    long numberOfLetters() {
	return this.seqNumber * this.seqLength;
    }

    /**
     * Checks if the number of sequences or their average length in the dataset
     * exceeds limits the values defined by this Limit
     * 
     * @param data
     * @return true if a limit is exceeded (what is the dataset is larger then
     *         the limit), false otherwise.
     */
    public boolean isExceeded(List<FastaSequence> data) {
	if (data == null) {
	    throw new NullPointerException(
		    "List of fasta sequences is expected!");
	}
	if (data.size() > this.seqNumber) {
	    return true;
	}
	if (this.seqLength != 0) {
	    if ((long) getAvgSeqLength() * data.size() > numberOfLetters()) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Calculates an average sequence length of the dataset
     * 
     * @param data
     * @return an average sequence length in the input dataset
     */
    public static int getAvgSequenceLength(List<FastaSequence> data) {
	long length = 0;
	for (FastaSequence seq : data) {
	    length += seq.getLength();
	}
	return (int) (length / data.size());
    }

    void validate() {
	if (this.seqNumber < 1) {
	    throw new AssertionError(
		    "Maximum number of sequences must be defined and be positive! Set value is: "
			    + this.seqNumber);
	}
	if (this.seqLength != 0 && this.seqLength < 1) {
	    throw new AssertionError(
		    "Average sequence length must be positive! Set value is: "
			    + this.seqLength);
	}
    }
}
